"""
Script for plotting loss and accuracy from chainer logs

usage:
	> python -m chainer_addon.utils.plot <logfile> [-k <key1> <key2> ... <keyn> ]
"""

import sys, argparse

parser = argparse.ArgumentParser(description = "Plot loss and accuracy from chainer logs")
parser.add_argument("logfile", type=str, help="chainer log file to parse")
parser.add_argument("--key","-k", type=str, default=["main/accuracy", "main/loss"], nargs="+", help="key of the loss and accuracy values")
parser.add_argument("--fit", action="store_true", help="plot fitted graphs")
parser.add_argument("--all_in_one", "-aio", action="store_true", help="plot all graphs in one")

from os.path import join
from scipy.optimize import curve_fit
import simplejson as json, numpy as np
import matplotlib.pyplot as plt

def func(x, a, b, c):
	return a * np.exp(-b * x) + c

def non_zero(idxs, values):
	return list(zip(*[(idx, val) for idx, val in zip(idxs, values) if val != 0]))

def main(args):
	log_file = json.load(open(args.logfile))#[:200]
	data = {key: [] for key in args.key}
	validation_data = {key: [] for key in args.key}
	for log in log_file:
		for key in args.key:
			data[key].append(log.get(key, 0))
			if "validation" in key: continue
			val_key = "validation/{}".format(key)
			if val_key in log:
				validation_data[key].append(log.get(val_key, 0))


	for i, (key, values) in enumerate(data.items()):
		X = np.arange(len(values))

		if i == 0 or not args.all_in_one:
			fig = plt.figure(i)
			fig.canvas.set_window_title("{} - {}".format(args.logfile, key))
			ax = fig.add_subplot(1,1,1)
			ax.set_title(key if not args.all_in_one else "graphs")

		if (np.array(values) != 0).any():
			ax.plot(X, values, label=key)

		if args.fit:
			coefs = curve_fit(func, X, data)[0]
			ax.plot(X, [func(x, *coefs) for x in X], "r--")

		if validation_data[key]:
			ax.plot(X, validation_data[key], label=key + " val")

		ax.legend()

	plt.show()



main(parser.parse_args())





import numpy as np
import chainer.links as L
import chainer.functions as F

from chainer.serializers import npz
from chainer.links.model.vision.vgg import prepare

from chainer_addons.models import PretrainedModelMixin

class VGG19Layers(PretrainedModelMixin, L.VGG16Layers):
	"""
		adds some additional layers to the original VGG16
		model definition. the last linear layer is also replaced
		if number of classes is given. this is useful in case of
		fine-tuning
	"""

	class meta:
		classifier_layers = ["fc6", "fc7", "fc8"]
		conv_map_layer = "conv5_4"
		n_conv_maps = 512
		mean = np.array([103.939, 116.779, 123.68], dtype=np.float32).reshape(3,1,1)

	@staticmethod
	def prepare(*args, **kw):
		return prepare(*args, **kw)

	@staticmethod
	def pooling(x):
		return F.max_pooling_2d(x, ksize=2, stride=2)

	def __init__(self, pretrained_model, n_classes=None):
		"""
			First we load vanilla VGG16 model, then additional layers are
			added and finally the model weights are loaded
		"""
		super(VGG19Layers, self).__init__(pretrained_model=None)

		self.add_link('conv3_4', L.Convolution2D(256, 256, 3, 1, 1))
		self.add_link('conv4_4', L.Convolution2D(512, 512, 3, 1, 1))
		self.add_link('conv5_4', L.Convolution2D(512, 512, 3, 1, 1))

		if n_classes is not None:
			self.fc8 = L.Linear(4096, n_classes, **init_args)

		self.functions = OrderedDict([
			('conv1_1',  [self.conv1_1, F.relu]),
			('conv1_2',  [self.conv1_2, F.relu]),
			('pool1',    [VGG19Layers.pooling]),
			('conv2_1',  [self.conv2_1, F.relu]),
			('conv2_2',  [self.conv2_2, F.relu]),
			('pool2',    [VGG19Layers.pooling]),
			('conv3_1',  [self.conv3_1, F.relu]),
			('conv3_2',  [self.conv3_2, F.relu]),
			('conv3_3',  [self.conv3_3, F.relu]),
			('conv3_4',  [self.conv3_4, F.relu]),
			('pool3',    [VGG19Layers.pooling]),
			('conv4_1',  [self.conv4_1, F.relu]),
			('conv4_2',  [self.conv4_2, F.relu]),
			('conv4_3',  [self.conv4_3, F.relu]),
			('conv4_4',  [self.conv4_4, F.relu]),
			('pool4',    [VGG19Layers.pooling]),
			('conv5_1',  [self.conv5_1, F.relu]),
			('conv5_2',  [self.conv5_2, F.relu]),
			('conv5_3',  [self.conv5_3, F.relu]),
			('conv5_4',  [self.conv5_4, F.relu]),
			('pool5',    [VGG19Layers.pooling]),
			('fc6',      [self.fc6, F.relu, F.dropout]),
			('fc7',      [self.fc7, F.relu, F.dropout]),
			('fc8',      [self.fc8]),
			('prob',     [F.softmax]),
		])

		npz.load_npz(pretrained_model, self)




"""
here you can find some model definitions or modificated versions
of present models in chainer (eg. VGG19)
"""
import chainer
import chainer.functions as F

from inspect import signature


def do_call(func, *args, **kw):
	sig = signature(func)
	train = kw.pop("train", True)
	if "train" in sig.parameters: 	kw["train"] = train
	elif "test" in sig.parameters:	kw["test"]  = not train

	return func(*args, **kw)

class Classifier(chainer.Chain):
	"""
		model wrapper, that is adapted for the DSL of
		the pretrained models in chainer
	"""
	def __init__(self, model, layer_name):
		super(Classifier, self).__init__(
			model=model
		)
		self.layer_name = layer_name

	def __call__(self, X, y, train=True):
		activations = do_call(self.model,
			X, layers=[self.layer_name], train=train)

		pred = activations[self.layer_name]

		loss, accu = F.softmax_cross_entropy(pred, y), F.accuracy(pred, y)
		chainer.report({
			"loss": loss.data,
			"accuracy": accu.data,
		}, self)
		return loss

class PretrainedModelMixin(object):
	def __call__(self, X, layer_name, train=True):
		super_call = super(PretrainedModelMixin, self).__call__
		return do_call(super_call, X, layers=[layer_name], train=train)[layer_name]

	def __init__(self, pretrained_model):
		return super(PretrainedModelMixin, self).__init__(pretrained_model)




from chainer_addons.models.vgg import VGG19Layers



#!/usr/bin/env python

import os
import pkg_resources
import sys

from setuptools import setup


install_requires = [line.strip() for line in open("requirements.txt").readlines()]

setup(
	name='chainer_addons',
	version='0.1.0',
	description='Some addon scripts for the chainer framework',
	author='Dimitri Korsch',
	author_email='korschdima@gmail.com',
	# url='https://chainer.org/',
	license='MIT License',
	packages=[
		'chainer_addons',
		'chainer_addons.models',
		'chainer_addons.utils',
	],
	zip_safe=False,
	setup_requires=[],
	install_requires=install_requires,
	# tests_require=['mock', 'nose'],
)
